from .EventDBPropertyTableHandler import makeEventDBPropertyTupleHandler
from .SettingPropertyHandler import makeSettingPropertyHandler
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler


def makeAdminBackendHandlers(
    tupleObservable: TupleDataObservableHandler, dbSessionCreator
):
    yield makeEventDBPropertyTupleHandler(tupleObservable, dbSessionCreator)

    yield makeSettingPropertyHandler(dbSessionCreator)



import logging
from datetime import datetime

from twisted.internet.defer import Deferred, inlineCallbacks
from vortex.Payload import Payload
from vortex.SerialiseUtil import ISO8601
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_base.storage.DbConnection import DbSessionCreator
from peek_plugin_base.storage.RunPyInPg import runPyInPg
from peek_plugin_eventdb.tuples import loadPublicTuples
from peek_plugin_eventdb.tuples.EventDBEventTuple import EventDBEventTuple

logger = logging.getLogger(__name__)


class EventDBEventTupleProvider(TuplesProviderABC):
    def __init__(self, dbSessionCreator: DbSessionCreator):
        self._dbSessionCreator = dbSessionCreator

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Deferred:
        # Uncomment this to print the SQL being created
        #
        # criterias = tupleSelector..get('criteria', [])
        # newestDateTime = tupleSelector..get('newestDateTime')
        # oldestDateTime = tupleSelector..get('oldestDateTime')
        #
        # sql = self._makeSql(criterias, 0, newestDateTime, oldestDateTime)
        # logger.info(sql)
        logger.debug(tupleSelector.toJsonStr())
        return (
            yield runPyInPg(
                logger,
                self._dbSessionCreator,
                self._loadInPg,
                self._importTuples,
                filt=filt,
                tupleSelectorStr=tupleSelector.toJsonStr(),
            )
        ).encode()

    @classmethod
    def _importTuples(cls):
        # noinspection PyUnresolvedReferences
        from peek_plugin_eventdb.tuples import EventDBPropertyCriteriaTuple

        loadPublicTuples()

    @classmethod
    def _loadInPg(cls, plpy, filt: dict, tupleSelectorStr: str):
        tupleSelector = TupleSelector.fromJsonStr(tupleSelectorStr)
        tuples = cls.loadTuples(plpy, tupleSelector)

        payloadEnvelope = Payload(
            filt=filt, tuples=tuples
        ).makePayloadEnvelope()
        vortexMsg = payloadEnvelope.toVortexMsg()
        return vortexMsg.decode()

    @classmethod
    def loadTuples(cls, plpy, tupleSelector: TupleSelector):
        selector = tupleSelector.selector
        modelSetKey = selector.get("modelSetKey")
        singleCriterias = selector.get("singleCriterias", {})
        multiCriterias = selector.get("multiCriterias", {})
        newestDateTime = selector.get("newestDateTime")
        oldestDateTime = selector.get("oldestDateTime")
        alarmsOnly = selector.get("alarmsOnly")

        if not modelSetKey:
            raise Exception("modelSetKey is None")

        modelSetId = cls.getModelSetId(plpy, modelSetKey)

        sql = cls._makeSql(
            singleCriterias,
            multiCriterias,
            modelSetId,
            newestDateTime,
            oldestDateTime,
            alarmsOnly=alarmsOnly,
        )

        # TODO, We probably need some pagination.

        def convertDate(strIn: str) -> datetime:
            dateTime, offset = strIn.split("+")
            if "." not in dateTime:
                dateTime += ".000"
            if len(offset) == 2:
                offset += "00"
            return datetime.strptime("%s+%s" % (dateTime, offset), ISO8601)

        tuples = []

        cursor = plpy.cursor(sql)
        while True:
            rows = cursor.fetch(1000)
            if not rows:
                break
            for row in rows:
                tuples.append(
                    EventDBEventTuple(
                        dateTime=convertDate(row["dateTime"]),
                        key=row["key"],
                        value=row["value"],
                    )
                )

        return tuples

    @classmethod
    def getModelSetId(cls, plpy, modelSetKey) -> int:
        # Load in the ModelSet ID
        sql = (
            """
              SELECT id FROM pl_eventdb."EventDBModelSet" where key = '%s'
              """
            % modelSetKey
        )
        rows = plpy.execute(sql, 1)

        if not len(rows):
            raise Exception("ModelSet with key %s not found" % modelSetKey)

        modelSetId = rows[0]["id"]
        return modelSetId

    @classmethod
    def _makeSql(
        cls,
        singleCriterias,
        multiCriterias,
        modelSetId,
        newestDateTime,
        oldestDateTime,
        alarmsOnly,
    ):
        # Create the basic SQL
        sql = (
            """
            SELECT "dateTime", key, value
            FROM pl_eventdb."EventDBEvent"
            WHERE "modelSetId" = %s
            """
            % modelSetId
        )

        # Add the Alarms Only
        if alarmsOnly:
            sql += """     AND "isAlarm" = true \n"""

        # Add in the date time criteria
        if newestDateTime:
            sql += (
                """     AND "dateTime" <= timestamp with time zone '%s' \n"""
                % newestDateTime
            )

        if oldestDateTime:
            sql += (
                """     AND timestamp with time zone '%s' <= "dateTime" \n"""
                % oldestDateTime
            )

        for propKey, value in singleCriterias.items():
            if not value:
                continue

            assert isinstance(value, str), "Property value is not a str"

            sql += """     AND "value"->>'%s' ilike '%s' \n""" % (
                propKey,
                "%" + value + "%",
            )

        for propKey, values in multiCriterias.items():
            if not values:
                continue

            assert isinstance(values, list), "Property values is not a list"

            criSql = []
            for value in values:
                try:
                    quotedVal = int(value)

                except ValueError:
                    if value.lower() == "true":
                        quotedVal = "true"

                    elif value.lower() == "false":
                        quotedVal = "false"

                    else:
                        quotedVal = '"%s"' % value

                criSql.append(
                    """     "value" @> '{"%s":%s}'::jsonb """
                    % (propKey, quotedVal)
                )

            sql += """     AND (%s) \n""" % "\n OR ".join(criSql)

        sql += """
               ORDER BY "dateTime" DESC
               LIMIT 5000
               """

        return sql



"""fix_useforpopup

Peek Plugin Database Migration Script

Revision ID: a63487e16375
Revises: 0338181f05cf
Create Date: 2020-07-14 12:50:01.280980

"""

# revision identifiers, used by Alembic.
revision = "a63487e16375"
down_revision = "0338181f05cf"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.alter_column(
        "EventDBProperty",
        "useForPopup",
        new_column_name="useForPopup_",
        schema="pl_eventdb",
    )

    op.add_column(
        "EventDBProperty",
        sa.Column("useForPopup", sa.Boolean()),
        schema="pl_eventdb",
    )

    op.execute(
        """UPDATE "pl_eventdb"."EventDBProperty"
                  SET "useForPopup" = ("useForPopup_" is not null) """
    )

    op.drop_column("EventDBProperty", "useForPopup_", schema="pl_eventdb")


def downgrade():
    pass



import logging
from datetime import datetime
from typing import Optional, List, Tuple

from sqlalchemy.dialects import postgresql
from vortex.Payload import Payload
from vortex.Tuple import TUPLE_TYPES_BY_NAME

from peek_plugin_eventdb._private.storage.EventDBModelSetTable import (
    EventDBModelSetTable,
)
from peek_plugin_eventdb.tuples import loadPublicTuples
from peek_plugin_eventdb.tuples.EventDBEventTuple import EventDBEventTuple

logger = logging.getLogger(__name__)


class EventDBImportEventsInPgTask:
    """EventDB Import In PostGreSQL Tasks

    The methods in this class are run in the databases plpython extension.
    """

    @classmethod
    def importEvents(
        cls, plpy, modelSetKey: str, eventsEncodedPayload: bytes
    ) -> Tuple[int, Optional[datetime], Optional[datetime]]:
        if EventDBEventTuple.tupleName() not in TUPLE_TYPES_BY_NAME:
            loadPublicTuples()

        # Reconstruct the data
        events = Payload().fromEncodedPayload(eventsEncodedPayload).tuples
        if not events:
            return 0, None, None

        dates = [e.dateTime for e in events]
        maxDate = max(dates)
        minDate = min(dates)

        # Get the model set id
        modelSetId = cls.getModelSetId(plpy, modelSetKey, createIfMissing=True)

        # Now insert the events
        cls._loadEvents(plpy, events, modelSetId)

        return len(events), maxDate, minDate

    @classmethod
    def deleteEvents(cls, plpy, modelSetKey: str, eventKeys: List[str]) -> int:
        # Get the model set id
        modelSetId = cls.getModelSetId(plpy, modelSetKey, createIfMissing=False)
        if modelSetId is None:
            plpy.debug("ModelSet with key %s doesn't exist" % modelSetKey)
            return 0

        # Now delete the events
        cls._deleteEvents(plpy, modelSetId, eventKeys)

        return len(eventKeys)

    @classmethod
    def updateAlarmFlags(
        cls, plpy, modelSetKey: str, eventKeys: List[str], alarmFlag: bool
    ) -> int:
        # Get the model set id
        modelSetId = cls.getModelSetId(plpy, modelSetKey, createIfMissing=False)
        if modelSetId is None:
            plpy.debug("ModelSet with key %s doesn't exist" % modelSetKey)
            return 0

        # Now insert the events
        cls._updateAlarmFlags(plpy, modelSetId, eventKeys, alarmFlag)

        return len(eventKeys)

    @classmethod
    def getModelSetId(cls, plpy, modelSetKey, createIfMissing) -> Optional[int]:
        msTbl = EventDBModelSetTable.__table__
        qryModelSetSql = str(
            msTbl.select()
            .where(msTbl.c.key == modelSetKey)
            .compile(
                dialect=postgresql.dialect(),
                compile_kwargs={"literal_binds": True},
            )
        )

        while True:
            rows = plpy.execute(qryModelSetSql, 1)
            if len(rows):
                return rows[0]["id"]

            if not createIfMissing:
                return None

            plpy.execute(
                """INSERT INTO pl_eventdb."EventDBModelSet"( key, name)
                            VALUES ('%s', '%s')"""
                % (modelSetKey, modelSetKey)
            )

    @classmethod
    def _loadEvents(cls, plpy, events, modelSetId: int):
        keys = list(set([e.key for e in events if e.key]))
        if keys:
            cls._deleteEvents(plpy, modelSetId, keys)

        plan = plpy.prepare(
            """INSERT INTO pl_eventdb."EventDBEvent"
                                ("dateTime", "key", "isAlarm",
                                        "modelSetId", value)
                                VALUES ($1, $2, $3,
                                        $4, $5);""",
            ["timestamp with time zone", "text", "boolean", "integer", "jsonb"],
        )
        for event in events:
            plpy.execute(
                plan,
                [
                    event.dateTime,
                    event.key,
                    event.isAlarm,
                    modelSetId,
                    event.value,
                ],
            )

    @classmethod
    def _deleteEvents(cls, plpy, modelSetId: int, eventKeys: List[str]):
        sql = """DELETE FROM pl_eventdb."EventDBEvent"
                 WHERE key in (%s)
                    AND "modelSetId" = %s;"""
        sql %= (", ".join("'%s'" % k for k in eventKeys), modelSetId)
        plpy.execute(sql)

    @classmethod
    def _updateAlarmFlags(
        cls, plpy, modelSetId: int, eventKeys: List[str], alarmFlag: bool
    ):
        sql = """UPDATE pl_eventdb."EventDBEvent"
                 SET "isAlarm" = %(isAlarm)s
                 WHERE key in (%(keys)s)
                    AND "modelSetId" = %(modelSetId)s;"""
        sql %= dict(
            modelSetId=modelSetId,
            keys=", ".join("'%s'" % k for k in eventKeys),
            isAlarm=str(alarmFlag).lower(),
        )
        plpy.execute(sql)



import logging

from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_base.storage.DbConnection import DbSessionCreator
from peek_plugin_eventdb._private.storage.EventDBModelSetTable import (
    EventDBModelSetTable,
)

logger = logging.getLogger(__name__)


class EventDBModelSetTableTupleProvider(TuplesProviderABC):
    def __init__(self, dbSessionCreator: DbSessionCreator):
        self._dbSessionCreator = dbSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Deferred:
        dbSession = self._dbSessionCreator()
        try:
            tuples = dbSession.query(EventDBModelSetTable).all()

        finally:
            dbSession.close()

        payloadEnvelope = Payload(
            filt=filt, tuples=tuples
        ).makePayloadEnvelope()
        vortexMsg = payloadEnvelope.toVortexMsg()
        return vortexMsg



import logging

logger = logging.getLogger(__name__)


class EventDBController(object):
    def __init__(self, dbSessionCreator):
        self._dbSessionCreator = dbSessionCreator

    def shutdown(self):
        pass



import logging

from peek_abstract_chunked_index.private.server.controller.ACIProcessorStatusControllerABC import (
    ACIProcessorStatusControllerABC,
)
from peek_plugin_eventdb._private.tuples.AdminStatusTuple import (
    AdminStatusTuple,
)

logger = logging.getLogger(__name__)


class AdminStatusController(ACIProcessorStatusControllerABC):
    NOTIFY_PERIOD = 2.0

    _StateTuple = AdminStatusTuple



from typing import Optional, List

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_eventdb._private.PluginNames import eventdbTuplePrefix
from peek_plugin_eventdb.tuples.EventDBPropertyValueTuple import (
    EventDBPropertyValueTuple,
)


@addTupleType
class EventDBPropertyTuple(Tuple):
    """Event DB Property Tuple

    This tuple stores the name of a property in the alarm / event that the user
    can filter on.

    // modelSetKey: The model that this property applies to.
    modelSetKey: string;

    // key: The unique id of this property within this model set, it must match
    //         the value of a key in the EventDBEventTuple.value json object.
    key: string;

    // name: The name to display to the user in a UI.
    name: string;

    // order: The order of this field
    order: number;

    // comment: The tooltip to display to the user in a UI.
    comment: string | null;

    // useForFilter: Can the user type free text in this field ?
    useForFilter: boolean | null;

    // useForDisplay: Can the user choose to see this
    useForDisplay: boolean | null;

    // useForPopup: Should this ID be used for popping up the DocDB
    useForPopup: boolean | null;

    // FOR DISPLAY
    // displayByDefaultOnSummaryView: Is this field visible by default when showing a
    //      summary alarm / event list.
    displayByDefaultOnSummaryView: boolean | null;

    // FOR DISPLAY
    // displayByDefaultOnDetailView: Is this field visible by default when showing a
    //      details/full alarm / event list.
    displayByDefaultOnDetailView: boolean | null;

    // FOR FILTER
    // showFilterAs: How should the filter be displayed to the user.
    showFilterAs: number | null;

    // FOR FILTER and FOR DISPLAY
    values: EventDBPropertyValueTuple[] | null;

    """

    __tupleType__ = eventdbTuplePrefix + "EventDBPropertyTuple"

    SHOW_FILTER_AS_FREE_TEXT = 1
    SHOW_FILTER_SELECT_MANY = 2
    SHOW_FILTER_SELECT_ONE = 3

    modelSetKey: str = TupleField()
    key: str = TupleField()
    name: str = TupleField()
    order: int = TupleField()
    comment: Optional[str] = TupleField()

    useForFilter: Optional[bool] = TupleField()
    useForDisplay: Optional[bool] = TupleField()
    useForPopup: Optional[bool] = TupleField()

    displayByDefaultOnSummaryView: Optional[bool] = TupleField()
    displayByDefaultOnDetailView: Optional[bool] = TupleField()

    showFilterAs: Optional[int] = TupleField()

    values: Optional[List[EventDBPropertyValueTuple]] = TupleField()



"""add color

Peek Plugin Database Migration Script

Revision ID: 0338181f05cf
Revises: af33359d5854
Create Date: 2020-06-06 20:06:52.966899

"""

# revision identifiers, used by Alembic.
revision = "0338181f05cf"
down_revision = "af33359d5854"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.add_column(
        "EventDBPropertyValue",
        sa.Column("color", sa.String()),
        schema="pl_eventdb",
    )


def downgrade():
    op.drop_column("EventDBPropertyValue", "color", schema="pl_eventdb")



from peek_plugin_eventdb._private.server.EventDBReadApi import EventDBReadApi
from peek_plugin_eventdb._private.server.EventDBWriteApi import EventDBWriteApi
from peek_plugin_eventdb._private.server.controller.EventDBController import (
    EventDBController,
)
from peek_plugin_eventdb._private.server.controller.EventDBImportController import (
    EventDBImportController,
)
from peek_plugin_eventdb.server.EventDBApiABC import EventDBApiABC
from peek_plugin_eventdb.server.EventDBReadApiABC import EventDBReadApiABC
from peek_plugin_eventdb.server.EventDBWriteApiABC import EventDBWriteApiABC


class EventDBApi(EventDBApiABC):
    def __init__(self):
        self._readApi = EventDBReadApi()
        self._writeApi = EventDBWriteApi()

    def setup(
        self,
        eventdbController: EventDBController,
        eventdbImportController: EventDBImportController,
        dbSessionCreator,
        dbEngine,
    ):
        self._readApi.setup(
            eventdbController=eventdbController,
            dbSessionCreator=dbSessionCreator,
            dbEngine=dbEngine,
        )

        self._writeApi.setup(
            eventdbController=eventdbController,
            eventdbImportController=eventdbImportController,
            readApi=self._readApi,
            dbSessionCreator=dbSessionCreator,
            dbEngine=dbEngine,
        )

    def shutdown(self):
        self._readApi.shutdown()
        self._writeApi.shutdown()

        self._readApi = None
        self._writeApi = None

    @property
    def writeApi(self) -> EventDBWriteApiABC:
        return self._writeApi

    @property
    def readApi(self) -> EventDBReadApiABC:
        return self._readApi



from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Index
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import orm
from sqlalchemy.orm import relationship
from vortex.Tuple import Tuple
from vortex.Tuple import addTupleType

from peek_plugin_eventdb._private.PluginNames import eventdbTuplePrefix
from .DeclarativeBase import DeclarativeBase
from .EventDBPropertyTable import EventDBPropertyTable
from ...tuples.EventDBPropertyValueTuple import EventDBPropertyValueTuple


@addTupleType
class EventDBPropertyValueTable(DeclarativeBase, Tuple):
    __tablename__ = "EventDBPropertyValue"
    __tupleType__ = eventdbTuplePrefix + "EventDBPropertyValueTable"

    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False)
    value = Column(String, nullable=False)
    color = Column(String)
    comment = Column(String)

    propertyId = Column(
        Integer,
        ForeignKey("EventDBProperty.id", ondelete="CASCADE"),
        nullable=False,
    )
    property = relationship(EventDBPropertyTable, backref="values")

    __table_args__ = (
        Index("idx_EventDBPropVal_name", propertyId, name, unique=True),
        Index("idx_EventDBPropVal_value", propertyId, value, unique=True),
    )

    @orm.reconstructor
    def __init__(self, **kwargs):
        Tuple.__init__(self, **kwargs)

    def toTuple(self) -> EventDBPropertyValueTuple:
        return EventDBPropertyValueTuple(
            name=self.name,
            value=self.value,
            color=self.color,
            comment=self.comment,
        )



"""initial tables

Peek Plugin Database Migration Script

Revision ID: 4ea424ad3883
Revises:
Create Date: 2020-05-24 14:18:31.113373

"""

# revision identifiers, used by Alembic.
revision = "4ea424ad3883"
down_revision = None
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "EventDBModelSet",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("propsJson", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("key"),
        sa.UniqueConstraint("name"),
        schema="pl_eventdb",
    )
    op.create_table(
        "Setting",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("name", sa.String(length=50), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_eventdb",
    )
    op.create_table(
        "EventDBEvent",
        sa.Column("id", sa.BigInteger(), autoincrement=True, nullable=False),
        sa.Column("dateTime", sa.DateTime(timezone=True), nullable=False),
        sa.Column(
            "value", postgresql.JSONB(astext_type=sa.Text()), nullable=False
        ),
        sa.Column("key", sa.String(), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"],
            ["pl_eventdb.EventDBModelSet.id"],
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("id", "dateTime"),
        schema="pl_eventdb",
    )
    op.create_table(
        "EventDBProperty",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("order", sa.Integer(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("useForFilter", sa.Boolean(), nullable=True),
        sa.Column("useForDisplay", sa.Boolean(), nullable=True),
        sa.Column("displayByDefaultOnSummaryView", sa.Boolean(), nullable=True),
        sa.Column("displayByDefaultOnDetailView", sa.Boolean(), nullable=True),
        sa.Column("showFilterAs", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["modelSetId"],
            ["pl_eventdb.EventDBModelSet.id"],
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_eventdb",
    )
    op.create_index(
        "idx_EventDBProp_name",
        "EventDBProperty",
        ["modelSetId", "key"],
        unique=True,
        schema="pl_eventdb",
    )
    op.create_index(
        "idx_EventDBProp_value",
        "EventDBProperty",
        ["modelSetId", "name"],
        unique=True,
        schema="pl_eventdb",
    )
    op.create_table(
        "SettingProperty",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("settingId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(length=50), nullable=False),
        sa.Column("type", sa.String(length=16), nullable=True),
        sa.Column("int_value", sa.Integer(), nullable=True),
        sa.Column("char_value", sa.String(length=50), nullable=True),
        sa.Column("boolean_value", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(
            ["settingId"],
            ["pl_eventdb.Setting.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_eventdb",
    )
    op.create_index(
        "idx_SettingProperty_settingId",
        "SettingProperty",
        ["settingId"],
        unique=False,
        schema="pl_eventdb",
    )
    op.create_table(
        "EventDBPropertyValue",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("value", sa.String(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("propertyId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["propertyId"],
            ["pl_eventdb.EventDBProperty.id"],
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_eventdb",
    )
    op.create_index(
        "idx_EventDBPropVal_name",
        "EventDBPropertyValue",
        ["propertyId", "name"],
        unique=True,
        schema="pl_eventdb",
    )
    op.create_index(
        "idx_EventDBPropVal_value",
        "EventDBPropertyValue",
        ["propertyId", "value"],
        unique=True,
        schema="pl_eventdb",
    )
    # ### end Alembic commands ###

    # Convert the table to a timescale table
    # https://docs.timescale.com/latest/api#create_hypertable
    sql = """
        SELECT create_hypertable('pl_eventdb."EventDBEvent"', 'dateTime');
    """

    op.execute(sql)

    op.create_index(
        "idx_EventDBEvent_modelSetId_key",
        "EventDBEvent",
        ["modelSetId", "key"],
        unique=False,
        schema="pl_eventdb",
    )


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_EventDBPropVal_value",
        table_name="EventDBPropertyValue",
        schema="pl_eventdb",
    )
    op.drop_index(
        "idx_EventDBPropVal_name",
        table_name="EventDBPropertyValue",
        schema="pl_eventdb",
    )
    op.drop_table("EventDBPropertyValue", schema="pl_eventdb")
    op.drop_index(
        "idx_SettingProperty_settingId",
        table_name="SettingProperty",
        schema="pl_eventdb",
    )
    op.drop_table("SettingProperty", schema="pl_eventdb")
    op.drop_index(
        "idx_EventDBProp_value",
        table_name="EventDBProperty",
        schema="pl_eventdb",
    )
    op.drop_index(
        "idx_EventDBProp_name",
        table_name="EventDBProperty",
        schema="pl_eventdb",
    )
    op.drop_table("EventDBProperty", schema="pl_eventdb")
    op.drop_index(
        "idx_EventDBEvent_modelSetId",
        table_name="EventDBEvent",
        schema="pl_eventdb",
    )
    op.drop_index(
        "idx_EventDBEvent_key", table_name="EventDBEvent", schema="pl_eventdb"
    )
    op.drop_table("EventDBEvent", schema="pl_eventdb")
    op.drop_table("Setting", schema="pl_eventdb")
    op.drop_table("EventDBModelSet", schema="pl_eventdb")
    # ### end Alembic commands ###



import logging
from typing import List

from twisted.internet.defer import Deferred, inlineCallbacks

from peek_plugin_eventdb._private.server.EventDBReadApi import EventDBReadApi
from peek_plugin_eventdb._private.server.controller.EventDBController import (
    EventDBController,
)
from peek_plugin_eventdb._private.server.controller.EventDBImportController import (
    EventDBImportController,
)
from peek_plugin_eventdb.server.EventDBWriteApiABC import EventDBWriteApiABC

logger = logging.getLogger(__name__)


class EventDBWriteApi(EventDBWriteApiABC):
    def __init__(self):
        self._queueController = None
        self._eventdbController = None
        self._eventdbImportController = None
        self._readApi = None
        self._dbSessionCreator = None
        self._dbEngine = None

    def setup(
        self,
        eventdbController: EventDBController,
        eventdbImportController: EventDBImportController,
        readApi: EventDBReadApi,
        dbSessionCreator,
        dbEngine,
    ):
        self._eventdbController = eventdbController
        self._eventdbImportController = eventdbImportController
        self._readApi = readApi
        self._dbSessionCreator = dbSessionCreator
        self._dbEngine = dbEngine

    def shutdown(self):
        pass

    @inlineCallbacks
    def addEvents(
        self, modelSetKey: str, eventsEncodedPayload: str
    ) -> Deferred:
        if not eventsEncodedPayload:
            return

        yield self._eventdbImportController.importEvents(
            modelSetKey, eventsEncodedPayload
        )

    @inlineCallbacks
    def removeEvents(self, modelSetKey: str, eventKeys: List[str]) -> Deferred:
        if not eventKeys:
            return

        yield self._eventdbImportController.deleteEvents(modelSetKey, eventKeys)

    @inlineCallbacks
    def updateAlarmFlags(
        self, modelSetKey: str, eventKeys: List[str], alarmFlag: bool
    ) -> Deferred:
        if not eventKeys:
            return

        yield self._eventdbImportController.updateAlarmFlags(
            modelSetKey, eventKeys, alarmFlag
        )

    @inlineCallbacks
    def replaceProperties(
        self, modelSetKey: str, propertiesEncodedPayload: str
    ) -> Deferred:
        if not propertiesEncodedPayload:
            return

        yield self._eventdbImportController.replaceProperties(
            modelSetKey, propertiesEncodedPayload
        )



from typing import Union, List

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_eventdb._private.PluginNames import eventdbTuplePrefix
from peek_plugin_eventdb.tuples.EventDBPropertyTuple import EventDBPropertyTuple
from peek_plugin_eventdb.tuples.EventDBPropertyValueTuple import (
    EventDBPropertyValueTuple,
)


@addTupleType
class EventDBPropertyCriteriaTuple(Tuple):
    """Event DB Property Criteria Tuple

    This tuple stores the criteria of alarms and events to retrieve.

    """

    __tupleType__ = eventdbTuplePrefix + "EventDBPropertyCriteriaTuple"

    property: EventDBPropertyTuple = TupleField()
    value: Union[List[str], str] = TupleField()



eventdbFilt = {"plugin": "peek_plugin_eventdb"}
eventdbTuplePrefix = "peek_plugin_eventdb."
eventdbObservableName = "peek_plugin_eventdb"
eventdbActionProcessorName = "peek_plugin_eventdb"
eventdbTupleOfflineServiceName = "peek_plugin_eventdb"



from abc import ABCMeta
from abc import abstractmethod
from typing import List

from twisted.internet.defer import Deferred


class EventDBWriteApiABC(metaclass=ABCMeta):
    @abstractmethod
    def addEvents(
        self, modelSetKey: str, eventsEncodedPayload: str
    ) -> Deferred:
        """Add Events

        Add events to the EventDB.

        :param modelSetKey: The name of the model set for the EventDB
        :param eventsEncodedPayload: An encoded Payload containing a
            list of events to insert
        :return: A deferred that fires when the insert is complete
        :rtype: Deferred
        """

    @abstractmethod
    def removeEvents(self, modelSetKey: str, eventKeys: List[str]) -> Deferred:
        """Remove Events

        Remove events from the EventDB.

        :param modelSetKey: The name of the model set for the EventDB
        :param eventKeys: A list of event keys to remove
        :return: A deferred that fires when the removal is complete
        :rtype: Deferred
        """

    @abstractmethod
    def updateAlarmFlags(
        self, modelSetKey: str, eventKeys: List[str], alarmFlag: bool
    ) -> Deferred:
        """Update Alarm Flag

        Change the value of the alarm flag for alarms.

        :param modelSetKey: The name of the model set for the EventDB
        :param eventKeys: A list of event keys to update the alarm flag
        :param alarmFlag: The value to set the alarm flag to
        :return: A deferred that fires when the update is complete
        :rtype: Deferred
        """

    @abstractmethod
    def replaceProperties(
        self, modelSetKey: str, propertiesEncodedPayload: str
    ) -> Deferred:
        """Replace Properties

        Create or Replace the properties for a model set of events.

        :param modelSetKey: The name of the model set for the EventDB
        :param propertiesEncodedPayload: An encoded Payload containing a
            payload.tuples=List[EventDBPropertyTuple]
            list of events to insert
        :return: A deferred that fires when the create / update is complete
        :rtype: Deferred
        """



import logging

from sqlalchemy import BigInteger
from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import orm
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.sql.schema import ForeignKey
from sqlalchemy.sql.schema import Index
from vortex.Tuple import Tuple
from vortex.Tuple import addTupleType

from peek_plugin_eventdb._private.PluginNames import eventdbTuplePrefix
from .DeclarativeBase import DeclarativeBase

logger = logging.getLogger(__name__)


@addTupleType
class EventDBEventTable(DeclarativeBase, Tuple):
    __tablename__ = "EventDBEvent"
    __tupleType__ = eventdbTuplePrefix + "EventDBEventTable"

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    dateTime = Column(DateTime(True), primary_key=True, nullable=False)

    value = Column(JSONB, nullable=False)

    key = Column(String)

    isAlarm = Column(Boolean)

    modelSetId = Column(
        Integer,
        ForeignKey("EventDBModelSet.id", ondelete="CASCADE"),
        nullable=False,
    )

    __table_args__ = (
        Index("idx_EventDBEvent_modelSetId_key", modelSetId, key, unique=False),
    )

    @orm.reconstructor
    def __init__(self, **kwargs):
        Tuple.__init__(self, **kwargs)



from vortex.Tuple import Tuple, addTupleType

from peek_plugin_eventdb._private.PluginNames import eventdbTuplePrefix


@addTupleType
class EventDBEventTuple(Tuple):
    """Event DB Event Tuple

    This tuple stores a value of a key in the Event DB database.

    datetime: is a timezone aware datetime
    key: is the key of this alarm or event (Optional)
    value: is the value of this event.

    """

    __tupleType__ = eventdbTuplePrefix + "EventDBEventTuple"
    __slots__ = ("dateTime", "key", "isAlarm", "value")

    @classmethod
    def sqlCoreLoad(cls, row):
        return EventDBEventTuple(
            datetime=row.datetime,
            key=row.key,
            isAlarm=row.isAlarm,
            value=row.value,
        )



import logging

from txhttputil.site.BasicResource import BasicResource

from peek_plugin_base.server.PluginLogicEntryHookABC import (
    PluginLogicEntryHookABC,
)
from peek_plugin_base.server.PluginServerStorageEntryHookABC import (
    PluginServerStorageEntryHookABC,
)
from peek_plugin_eventdb._private.server.controller.EventDBController import (
    EventDBController,
)
from peek_plugin_eventdb._private.server.controller.EventDBImportController import (
    EventDBImportController,
)
from peek_plugin_eventdb._private.server.download_resources.DownloadEventsResource import (
    DownloadEventsResource,
)
from peek_plugin_eventdb._private.storage import DeclarativeBase
from peek_plugin_eventdb._private.storage.DeclarativeBase import (
    loadStorageTuples,
)
from peek_plugin_eventdb._private.tuples import loadPrivateTuples
from peek_plugin_eventdb.tuples import loadPublicTuples
from .EventDBApi import EventDBApi
from .TupleActionProcessor import makeTupleActionProcessorHandler
from .TupleDataObservable import makeTupleDataObservableHandler
from .admin_backend import makeAdminBackendHandlers
from .controller.AdminStatusController import AdminStatusController
from .controller.MainController import MainController

logger = logging.getLogger(__name__)


class LogicEntryHook(PluginLogicEntryHookABC, PluginServerStorageEntryHookABC):

    def __init__(self, *args, **kwargs):
        """ " Constructor"""
        # Call the base classes constructor
        PluginLogicEntryHookABC.__init__(self, *args, **kwargs)

        #: Loaded Objects, This is a list of all objects created when we start
        self._loadedObjects = []

        self._api = None

    def load(self) -> None:
        """Load

        This will be called when the plugin is loaded, just after the db is migrated.
        Place any custom initialiastion steps here.

        """
        self._api = EventDBApi()
        self._loadedObjects.append(self._api)

        loadStorageTuples()
        loadPrivateTuples()
        loadPublicTuples()
        logger.debug("Loaded")

    @property
    def dbMetadata(self):
        return DeclarativeBase.metadata

    def start(self):
        """Start

        This will be called when the plugin is loaded, just after the db is migrated.
        Place any custom initialiastion steps here.

        """
        # ----------------
        # create the Status Controller
        statusController = AdminStatusController()
        self._loadedObjects.append(statusController)

        # ----------------
        # Create the Tuple Observer
        tupleObservable = makeTupleDataObservableHandler(
            self.dbSessionCreator, statusController
        )
        self._loadedObjects.append(tupleObservable)

        # ----------------
        # Tell the status controller about the Tuple Observable
        statusController.setTupleObservable(tupleObservable)

        # ----------------
        # Initialise the handlers for the admin interface
        self._loadedObjects.extend(
            makeAdminBackendHandlers(tupleObservable, self.dbSessionCreator)
        )

        # ----------------
        # create the Main Controller
        mainController = MainController(
            dbSessionCreator=self.dbSessionCreator,
            tupleObservable=tupleObservable,
        )

        self._loadedObjects.append(mainController)

        # ----------------
        # Create the Action Processor
        self._loadedObjects.append(
            makeTupleActionProcessorHandler(mainController)
        )

        # ----------------
        # Create the EventDB controller
        eventdbController = EventDBController(self.dbSessionCreator)
        self._loadedObjects.append(eventdbController)

        # ----------------
        # Create the Import Controller
        eventdbImportController = EventDBImportController(
            self.dbSessionCreator, statusController, tupleObservable
        )
        self._loadedObjects.append(eventdbImportController)

        # ----------------
        # Initialise the API object that will be shared with other plugins
        self._api.setup(
            eventdbController=eventdbController,
            eventdbImportController=eventdbImportController,
            dbSessionCreator=self.dbSessionCreator,
            dbEngine=self.dbEngine,
        )

        # ----------------
        # Start the queue controller

        # noinspection PyTypeChecker
        eventdbImportController.setReadApi(self._api.readApi)
        eventdbImportController.start()

        downloadResource = BasicResource()
        eventsResource = DownloadEventsResource(self.dbSessionCreator)
        # noinspection PyTypeChecker
        downloadResource.putChild(b"events", eventsResource)

        self.platform.addServerResource(b"download", downloadResource)

        logger.debug("Started")

    def stop(self):
        """Stop

        This method is called by the platform to tell the peek app to shutdown and stop
        everything it's doing
        """
        # Shutdown and dereference all objects we constructed when we started
        while self._loadedObjects:
            self._loadedObjects.pop().shutdown()

        self._api = None

        logger.debug("Stopped")

    def unload(self):
        """Unload

        This method is called after stop is called, to unload any last resources
        before the PLUGIN is unlinked from the platform

        """
        logger.debug("Unloaded")

    @property
    def publishedServerApi(self) -> object:
        """Published Server API

        :return  class that implements the API that can be used by other Plugins on this
        platform service.
        """
        return self._api



from peek_plugin_base.storage.AlembicEnvBase import AlembicEnvBase

from peek_plugin_eventdb._private.storage import DeclarativeBase
from peek_plugin_eventdb._private.storage.DeclarativeBase import (
    loadStorageTuples,
)

loadStorageTuples()

alembicEnv = AlembicEnvBase(DeclarativeBase.DeclarativeBase.metadata)
alembicEnv.run()



from twisted.trial import unittest

from peek_plugin_eventdb._private.PluginNames import eventdbTuplePrefix


class MyTestCase(unittest.TestCase):
    def testPluginName(self):
        self.assertEqual(eventdbTuplePrefix, "peek_plugin_eventdb.")



from peek_plugin_eventdb._private.server.tuple_selector_mappers.NewEventTSUpdateMapper import (
    NewEventTSUpdateMapper,
)
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

from peek_plugin_base.storage.DbConnection import DbSessionCreator
from peek_plugin_eventdb._private.PluginNames import eventdbFilt
from peek_plugin_eventdb._private.PluginNames import eventdbObservableName
from peek_plugin_eventdb._private.server.controller.AdminStatusController import (
    AdminStatusController,
)
from peek_plugin_eventdb._private.server.tuple_providers.AdminStatusTupleProvider import (
    AdminStatusTupleProvider,
)
from peek_plugin_eventdb._private.server.tuple_providers.EventDBEventTupleProvider import (
    EventDBEventTupleProvider,
)
from peek_plugin_eventdb._private.server.tuple_providers.EventDBModelSetTableTupleProvider import (
    EventDBModelSetTableTupleProvider,
)
from peek_plugin_eventdb._private.server.tuple_providers.EventDBPropertyTupleProvider import (
    EventDBPropertyTupleProvider,
)
from peek_plugin_eventdb._private.storage.EventDBModelSetTable import (
    EventDBModelSetTable,
)
from peek_plugin_eventdb._private.tuples.AdminStatusTuple import (
    AdminStatusTuple,
)
from peek_plugin_eventdb.tuples.EventDBEventTuple import EventDBEventTuple
from peek_plugin_eventdb.tuples.EventDBPropertyTuple import EventDBPropertyTuple


def makeTupleDataObservableHandler(
    ormSessionCreator: DbSessionCreator,
    adminStatusController: AdminStatusController,
):
    """ " Make Tuple Data Observable Handler

    This method creates the observable object, registers the tuple providers and then
    returns it.

    :param adminStatusController:
    :param ormSessionCreator: A function that returns a SQLAlchemy session when called

    :return: An instance of :code:`TupleDataObservableHandler`

    """
    tupleObservable = TupleDataObservableHandler(
        observableName=eventdbObservableName, additionalFilt=eventdbFilt
    )

    # Add the tuple selector update mappers
    tupleObservable.addTupleSelectorUpdateMapper(NewEventTSUpdateMapper())

    # Admin Tuple Observers
    tupleObservable.addTupleProvider(
        AdminStatusTuple.tupleName(),
        AdminStatusTupleProvider(adminStatusController),
    )

    tupleObservable.addTupleProvider(
        EventDBModelSetTable.tupleName(),
        EventDBModelSetTableTupleProvider(ormSessionCreator),
    )

    # UI Tuple Observers
    tupleObservable.addTupleProvider(
        EventDBEventTuple.tupleName(),
        EventDBEventTupleProvider(ormSessionCreator),
    )

    tupleObservable.addTupleProvider(
        EventDBPropertyTuple.tupleName(),
        EventDBPropertyTupleProvider(ormSessionCreator),
    )
    return tupleObservable
